@@ -16,7 +16,7 @@
 #
 #  High level module for managing the lookup of R objects.
 #
-#  $Id: rpy.py,v 1.13 2004/12/23 00:01:26 warnes Exp $
+#  $Id: rpy.py,v 1.21 2005/07/20 05:38:03 warnes Exp $
 #
 #
 from __future__ import nested_scopes
@@ -24,6 +24,7 @@ from __future__ import nested_scopes
 import rpy_io
 import UserDict
 import time, os, sys
+import rpy_tools
 
 # If we cannot import Numeric, it should have been detected at
 # installation time and RPy should have been compiled properly. So,
@@ -33,36 +34,28 @@ try:
 except ImportError:
     pass
 
-# try to import rpy binary library using current path
-try:
-    import _rpy
-except ImportError:
-    if sys.platform == 'win32':
-        import win32api, win32con
+RHOME = rpy_tools.get_R_HOME()
+RVERSION  = rpy_tools.get_R_VERSION(RHOME)
+RVER = rpy_tools.get_R_VERSION_CODE(RVERSION)
 
-        hkey = win32api.RegOpenKeyEx( win32con.HKEY_LOCAL_MACHINE,
-                                      "Software\\R-core\\R",
-                                      0, win32con.KEY_QUERY_VALUE )
-        # get the base directory
-        rpath = win32api.RegQueryValueEx( hkey, "InstallPath" )[0]
-        win32api.RegCloseKey( hkey )
-        Rlib = os.path.join( rpath, 'bin', 'R.dll')
-        
-        # Now, load the library into the python address space and retry the
-        # import
-        win32api.LoadLibraryEx( Rlib, 0,
-                                win32con.LOAD_WITH_ALTERED_SEARCH_PATH )
-        import _rpy
-    else:
-        raise
+if sys.platform == 'win32':
+    # Load the R dll using the explicit path
+    Rlib = os.path.join( rpath, 'bin', 'R.dll')
+    win32api.LoadLibraryEx( Rlib, 0,
+                            win32con.LOAD_WITH_ALTERED_SEARCH_PATH )
+    
+# loading the version of rpy that matches the verison of R we're using
+exec("import _rpy%s as _rpy" % RVER)
 
 # Version
 from rpy_version import rpy_version
 
 # Symbolic names for conversion modes
-PROC_CONVERSION = 3
-CLASS_CONVERSION = 2
-BASIC_CONVERSION = 1
+TOP_CONVERSION = 4
+PROC_CONVERSION = 4
+CLASS_CONVERSION = 3
+BASIC_CONVERSION = 2
+VECTOR_CONVERSION = 1
 NO_CONVERSION = 0
 NO_DEFAULT = -1
 
@@ -170,11 +163,17 @@ proc_table = Dict_With_Mode(_rpy.__proc_table__)
 class R:
 
     def __init__(self):
+        _rpy.r_init()
         _rpy.set_mode(NO_DEFAULT)
         self.TRUE = self.__getitem__('T')
         self.FALSE = self.__getitem__('F')
+        self.NA = self('NA')
+        self.NAN = self('as.double(NA)')
+        self.helpfun = with_mode(NO_CONVERSION, self.__getitem__('help'))
+        self.help = self.__help__  # override r.help()
         
     def __getattr__(self, name):
+        if name.startswith('__') and name.endswith('__'): return object.__getattr__(name)
         if len(name) > 1 and name[-1] == '_' and name[-2] != '_':
             name = name[:-1]
         name = name.replace('__', '<-')
@@ -186,7 +185,28 @@ class R:
         return obj
 
     def __call__(self, s):
-        return r.eval(r.parse(text=s))
+        return self.eval(self.parse(text=s))
+
+    def __help__(self, *arg, **kw):
+        """
+        R's help funtion now returns an object of class help that
+        must be printed (by R) in order to be rendered.  This
+        function forces printing so that the user get the expected
+        behavior.
+        """
+        helpobj = self.helpfun(*arg, **kw)
+        self.print_(helpobj)
+
+    def __repr__(self):
+        Rver = self.__getitem__('R.version.string').as_py()
+        return "RPy version %s [%s]" % (rpy_version, Rver)
+
+    def __str__(self):
+        return repr(self)
+
+    def __cleanup__(self):
+        _rpy.r_cleanup()
+        del(self)
     
 # main instance
 r = R()
