@@ -16,7 +16,7 @@
  *
  *  Public API for calling R.
  *
- *  $Id: RPy.h,v 1.12 2004/10/23 01:55:53 warnes Exp $
+ *  $Id: RPy.h,v 1.15 2005/07/19 20:39:43 warnes Exp $
  *
  */
 
@@ -33,19 +33,31 @@
 #include <Rdefines.h>
 #include <Rinternals.h>
 #include <Rversion.h>
+#undef _POSIX_C_SOURCE
 #include <Python.h>
 #include <R_ext/Rdynload.h>
 #include <R_ext/eventloop.h>
 
 #ifdef _WIN32
 #define Win32  /* needed to get definition for UIMode */
-#include <Startup.h>
+#include <Rinterface.h>
 #undef Win32
 #include <Graphics.h>
 #else
-#include <Startup.h>
+#include <Rinterface.h>
 #endif 
 
+#include <Rdevices.h> /* must follow Graphics.h */
+
+
+/* Missing definitions from Rinterface.h or RStartup.h */
+# define CleanEd		Rf_CleanEd
+extern void CleanEd(void);
+extern int	R_CollectWarnings; 
+# define PrintWarnings		Rf_PrintWarnings
+extern void PrintWarnings(void);
+/****/
+
 #include <signal.h>
 
 #include "robjobject.h"
@@ -64,11 +76,12 @@ PyObject *to_Pyobj_table(SEXP);
 PyObject *to_Pyobj_with_mode(SEXP, int);
 
 #define NO_CONVERSION 0
-#define BASIC_CONVERSION 1
-#define CLASS_CONVERSION 2
-#define PROC_CONVERSION 3
+#define VECTOR_CONVERSION 1
+#define BASIC_CONVERSION 2
+#define CLASS_CONVERSION 3
+#define PROC_CONVERSION 4
 
-#define TOP_MODE 3
+#define TOP_MODE 4
 
 /* R Evaluation */
 SEXP do_eval_expr(SEXP e);
@@ -82,14 +95,14 @@ char *get_last_error_msg(void);
 
 /* For initializing R */
 extern int Rf_initEmbeddedR(int argc, char *argv[]);
-extern void init_io_routines(PyObject *);
+extern void init_io_routines(void);
 
 /* I/O functions */
-#ifdef _WIN32
-__declspec(dllimport) FILE *R_Outputfile;
-#else
+//#ifdef _WIN32
+//__declspec(dllimport) FILE *R_Outputfile;
+//#else
 extern FILE *R_Outputfile;
-#endif
+//#endif
 
 #ifdef _WIN32
 extern void RPy_WriteConsole(char *, int);
@@ -112,12 +125,6 @@ extern char *getRHOME(void);
 __declspec(dllimport) int UserBreak;
 #endif
 
-
-
-
-
-
-
 /* Setters for io functions */
 PyObject *set_output(PyObject *self, PyObject *args);
 PyObject *set_input(PyObject *self, PyObject *args);
@@ -150,4 +157,13 @@ extern PyObject *rpy_dict;
 void stop_events(void);
 void start_events(void);
 
+
+/* Translation functions */
+extern int to_Pyobj_proc(SEXP robj, PyObject **obj);
+extern int to_Pyobj_class(SEXP robj, PyObject **obj);
+extern int to_Pyobj_basic(SEXP robj, PyObject **obj);
+extern int to_Pyobj_vector(SEXP robj, PyObject **obj, int mode);
+
+
+
 #endif /* _RPY_H */
