@@ -16,7 +16,7 @@
  *
  *  Input/Output routines
  *
- *  $Id: io.c,v 1.6 2004/02/27 04:47:27 warnes Exp $
+ *  $Id: io.c,v 1.7 2005/07/14 06:37:58 warnes Exp $
  *
  */
 
@@ -33,7 +33,7 @@
        PyEval_ReleaseThread(tstate);}\
     } 
 
-PyObject *rpy_output, *rpy_input, *rpy_showfiles;
+PyObject *rpy_output=NULL, *rpy_input=NULL, *rpy_showfiles=NULL;
 
 /* Show the traceback of an exception which occurred in a I/O process,
    except when the error is a KeyboardInterrupt, in which case abort
@@ -122,7 +122,7 @@ RPy_ShowFiles(int nfile, char **file, char **headers,
   PyOS_sighandler_t old_int;
   int i;
 
-  if (!rpy_showfiles)
+  if (rpy_showfiles==NULL)
     return 0;
 
   old_int = PyOS_getsig(SIGINT);
@@ -237,7 +237,7 @@ get_showfiles(PyObject *self, PyObject *args)
 }
 
 void
-init_io_routines(PyObject *d)
+init_io_routines(void)
 #ifdef _WIN32
 { return; }
 #else
