@@ -16,7 +16,7 @@
  *
  *  Implementation of the module '_rpy' and the 'Robj' type.
  *
- *  $Id: rpymodule.c,v 1.20 2004/10/23 01:55:53 warnes Exp $
+ *  $Id: rpymodule.c,v 1.27 2005/07/20 05:38:03 warnes Exp $
  *
  */
 
@@ -28,6 +28,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+/* Local function definitions */
+DL_EXPORT(void) INIT_RPY(void);   /* Module initializer */
+static PyObject *r_init(void);    /* Class initializer */
+static PyObject *r_cleanup(void); /* Clean up R & release resources */
+
+
 /* Global objects */
 static SEXP get_item;
 static SEXP set_item;
@@ -118,30 +124,32 @@ Robj_tpnew(PyTypeObject *type, PyObject *args, PyObject *kwds)
 /* These are auxiliaries for a state machine for converting Python
    list to the coarsest R vector type */
 #define ANY_T 0
-#define INT_T 1
-#define FLOAT_T 2
-#define COMPLEX_T 3
-#define STRING_T 4
+#define BOOL_T 1
+#define INT_T 2
+#define FLOAT_T 3
+#define COMPLEX_T 4
+#define STRING_T 5
+#define ROBJ_T 6
 
 static int
 type_to_int(PyObject *obj)
 {
-  int r = 0;
-
-  if (PyInt_Check(obj))
-    r = INT_T;
-  if (PyFloat_Check(obj))
-    r = FLOAT_T;
-  if (PyComplex_Check(obj))
-    r = COMPLEX_T;
-
-  if (PyNumber_Check(obj))
-    return r;
-
-  if (PyString_Check(obj))
+  if (PyBool_Check(obj))
+    return BOOL_T;
+  else if (PyInt_Check(obj))
+    return INT_T;
+  else if (PyFloat_Check(obj))
+    return FLOAT_T;
+  else if (PyComplex_Check(obj))
+    return COMPLEX_T;
+  else if (PyNumber_Check(obj))
+    return ANY_T;
+  else if (PyString_Check(obj))
     return STRING_T;
-
-  return ANY_T;
+  else if (Robj_Check(obj))
+    return ROBJ_T;
+  else
+    return ANY_T;
 }
 
 /* Make a R list or vector from a Python sequence */
@@ -151,11 +159,25 @@ seq_to_R(PyObject *obj)
   PyObject *it;
   SEXP robj, rit;
   int i, len, state;
-  int fsm[5][5] = {{0, 0, 0, 0, 0},
-                   {0, 1, 2, 3, 0},
-                   {0, 2, 2, 3, 0},
-                   {0, 3, 3, 3, 0},
-                   {0, 0, 0, 0, 4}};
+
+  /* This matrix defines what mode a vector should take given what
+     it already contains and a new item
+  
+     E.g. Row 0 indicates that if we've seen an any, the vector will
+     always remain an any.  Row 3 indicates that if we've seen a
+     float, then seeing an boolean, integer, or float will preserve
+     the vector as a float vector, while seeing a string or an Robj will
+     convert it into an any vector.
+  */
+  int fsm[7][7] = {
+    {0, 0, 0, 0, 0, 0, 0}, // any
+    {0, 1, 2, 3, 4, 0, 0}, // bool
+    {0, 2, 2, 3, 4, 0, 0}, // int
+    {0, 3, 3, 3, 4, 0, 0}, // float
+    {0, 4, 4, 4, 4, 0, 0}, // complex
+    {0, 0, 0, 0, 0, 5, 0}, // string
+    {0, 0, 0, 0, 0, 0, 6}  // RObj
+  };
   
   len = PySequence_Length(obj);
   if (len == 0)
@@ -175,23 +197,36 @@ seq_to_R(PyObject *obj)
 
     if (!(rit = to_Robj(it)))
       goto exception;
+    
     SET_VECTOR_ELT(robj, i, rit);
     Py_XDECREF(it);
   }
 
-  if (state == INT_T)
-    robj = AS_INTEGER(robj);
-  if (state == FLOAT_T)
-    robj = AS_NUMERIC(robj);
-  if (state == COMPLEX_T)
-    robj = AS_COMPLEX(robj);
-  if (state == STRING_T)
-    robj = AS_CHARACTER(robj);
-
+  switch(state)
+    {
+    case INT_T:
+      robj = AS_INTEGER(robj);
+      break;
+    case BOOL_T:
+      robj = AS_LOGICAL(robj);
+      break;
+    case FLOAT_T:
+      robj = AS_NUMERIC(robj);
+      break;
+    case COMPLEX_T:
+      robj = AS_COMPLEX(robj);
+      break;
+    case STRING_T:
+      robj = AS_CHARACTER(robj);
+      break;
+    case ROBJ_T: /* We should never see an Robj here */
+      robj = NULL;
+    }
+  
   UNPROTECT(1);
   return robj;
 
- exception:
+exception:
   Py_XDECREF(it);
   UNPROTECT(1);
   return NULL;
@@ -321,50 +356,66 @@ to_Robj(PyObject *obj)
     do_decref = 1;
   }
   PyErr_Clear();
-    
-  if (Robj_Check(obj)) {
-    PROTECT(robj = ((RobjObject *)obj)->R_obj);
-
-  } else if (PyInt_Check(obj)) {
-    PROTECT(robj = NEW_INTEGER(1));
-    INTEGER_DATA(robj)[0] = PyInt_AsLong(obj);
-
-  } else if (PyFloat_Check(obj)) {
-    PROTECT(robj = NEW_NUMERIC(1));
-    NUMERIC_DATA(robj)[0] = PyFloat_AsDouble(obj);
-
-  } else if (PyComplex_Check(obj)) {
-    PROTECT(robj = NEW_COMPLEX(1));
-    c = PyComplex_AsCComplex(obj);
-    COMPLEX_DATA(robj)[0].r = c.real;
-    COMPLEX_DATA(robj)[0].i = c.imag;
-
-  } else if (PyString_Check(obj)) {
-    PROTECT(robj = NEW_STRING(1));
-    SET_STRING_ELT(robj, 0, COPY_TO_USER_STRING(PyString_AsString(obj)));
 
+  
+  if (Robj_Check(obj))
+    {
+      PROTECT(robj = ((RobjObject *)obj)->R_obj);
+    }
+  else if (PyBool_Check(obj))
+    {
+      PROTECT(robj = NEW_LOGICAL(1));
+      LOGICAL_DATA(robj)[0] = (Py_True==obj);
+    }
+  else if (PyInt_Check(obj))
+    {
+      PROTECT(robj = NEW_INTEGER(1));
+      INTEGER_DATA(robj)[0] = PyInt_AsLong(obj);
+    }
+  else if (PyFloat_Check(obj)) 
+    {
+      PROTECT(robj = NEW_NUMERIC(1));
+      NUMERIC_DATA(robj)[0] = PyFloat_AsDouble(obj);
+    }
+  else if (PyComplex_Check(obj)) 
+    {
+      PROTECT(robj = NEW_COMPLEX(1));
+      c = PyComplex_AsCComplex(obj);
+      COMPLEX_DATA(robj)[0].r = c.real;
+      COMPLEX_DATA(robj)[0].i = c.imag;
+    }
+  else if (PyString_Check(obj)) 
+    {
+      PROTECT(robj = NEW_STRING(1));
+      SET_STRING_ELT(robj, 0, COPY_TO_USER_STRING(PyString_AsString(obj)));
 #ifdef WITH_NUMERIC
-  } else if (PyArray_Check(obj)) {
-    PROTECT(robj = to_Rarray(obj));
+    }
+  else if (PyArray_Check(obj)) 
+    {
+      PROTECT(robj = to_Rarray(obj));
 #endif
+    }
+  else if ((PySequence_Check(obj)) &&
+           (PySequence_Size(obj) >= 0)) 
+    {
+      PROTECT(robj = seq_to_R(obj));      /* No labels */
+    }
+  else if ((PyMapping_Check(obj)) &&
+             (PyMapping_Size(obj) >= 0)) 
+    {
+      PROTECT(robj = dict_to_R(obj));
+    }
+  else 
+    {
+      PyErr_Format(RPyExc_Exception, "cannot convert from type '%s'",
+                   obj->ob_type->tp_name);
+      PROTECT(robj = NULL);       /* Protected to avoid stack inbalance */
+    }
 
-  } else if ((PySequence_Check(obj)) &&
-             (PySequence_Size(obj) >= 0)) {
-    PROTECT(robj = seq_to_R(obj));      /* No labels */
-
-  } else if ((PyMapping_Check(obj)) &&
-             (PyMapping_Size(obj) >= 0)) {
-    PROTECT(robj = dict_to_R(obj));
-
-  } else {
-    PyErr_Format(RPyExc_Exception, "cannot convert from type '%s'",
-                 obj->ob_type->tp_name);
-    PROTECT(robj = NULL);       /* Protected to avoid stack inbalance */
-  }
-
-  if (do_decref) {
-    Py_DECREF(obj);
-  }
+  if (do_decref) 
+    {
+      Py_DECREF(obj);
+    }
   UNPROTECT(1);
   return robj;
 }
@@ -429,7 +480,7 @@ ltranspose(PyObject *list, int *dims, int *strides,
       
 /* Convert a Python list to a Python array (in the form of
  * list of lists of ...) */
-PyObject *
+static PyObject *
 to_PyArray(PyObject *obj, int *dims, int l)
 {
   PyObject *list;
@@ -453,7 +504,7 @@ to_PyArray(PyObject *obj, int *dims, int l)
 
 /* Convert a Python sequence to a Numeric array */
 #ifdef WITH_NUMERIC
-PyObject *
+static PyObject *
 to_PyNumericArray(PyObject *seq, SEXP dim)
 {
   PyObject *array, *ret, *dims, *it;
@@ -491,13 +542,37 @@ to_PyNumericArray(PyObject *seq, SEXP dim)
 }
 #endif
 
-/* Convert a R object to a 'basic' Python object (mode 1) */
+/* Convert an R object to a 'basic' Python object (mode 2) */
+/* NOTE: R vectors of length 1 will yield a python scalar */
 int
 to_Pyobj_basic(SEXP robj, PyObject **obj)
+{
+  int status;
+  PyObject *tmp;
+  
+  status = to_Pyobj_vector(robj, &tmp, BASIC_CONVERSION);
+
+  if(status==1 && PyList_Check(tmp) && PyList_Size(tmp) == 1)
+    {
+      *obj = PyList_GetItem(tmp, 0);
+      Py_XINCREF(*obj);
+      Py_DECREF(tmp);
+    }
+  else
+    *obj = tmp;
+  
+  return status;
+}
+
+
+/* Convert an R object to a 'vector' Python object (mode 1) */
+/* NOTE: R vectors of length 1 will yield a python list of length 1*/
+int
+to_Pyobj_vector(SEXP robj, PyObject **obj, int mode)
 {
   PyObject *it, *tmp;
   SEXP names, dim;
-  int len, *integers, i;
+  int len, *integers, i, type;
   char *strings, *thislevel;
   double *reals;
   Rcomplex *complexes;
@@ -516,49 +591,72 @@ to_Pyobj_basic(SEXP robj, PyObject **obj)
 
   len = GET_LENGTH(robj);
   tmp = PyList_New(len);
-
+  type = TYPEOF(robj);
+  
   for (i=0; i<len; i++) {
-    switch (TYPEOF(robj)) {
-    case LGLSXP:
-    case INTSXP:
-      integers = INTEGER(robj);
-      if(isFactor(robj)) {
-        thislevel = CHAR(STRING_ELT(GET_LEVELS(robj), integers[i]-1));
-        if (!(it = PyString_FromString(thislevel)))
+    switch (type)
+      {
+      case LGLSXP:
+         integers = INTEGER(robj);
+         if(integers[i]==NA_INTEGER) /* watch out for NA's */
+           {
+             if (!(it = PyInt_FromLong(integers[i])))
+             return -1;
+             //it = Py_None;
+           }
+         else if (!(it = PyBool_FromLong(integers[i])))
+           return -1;
+         break;
+      case INTSXP:
+        integers = INTEGER(robj);
+        if(isFactor(robj)) {
+          /* Watch for NA's! */
+          if(integers[i]==NA_INTEGER)
+            it = PyString_FromString(CHAR(NA_STRING));
+          else
+            {
+              thislevel = CHAR(STRING_ELT(GET_LEVELS(robj), integers[i]-1));
+              if (!(it = PyString_FromString(thislevel)))
+                return -1;
+            }
+        }
+        else {
+          if (!(it = PyInt_FromLong(integers[i])))
+            return -1;
+        }
+        break;
+      case REALSXP:
+        reals = REAL(robj);
+        if (!(it = PyFloat_FromDouble(reals[i])))
           return -1;
-      }
-      else {
-        if (!(it = PyInt_FromLong(integers[i])))
+        break;
+      case CPLXSXP:
+        complexes = COMPLEX(robj);
+        if (!(it = PyComplex_FromDoubles(complexes[i].r,
+                                         complexes[i].i)))
           return -1;
-      }
-      break;
-    case REALSXP:
-      reals = REAL(robj);
-      if (!(it = PyFloat_FromDouble(reals[i])))
-        return -1;
-      break;
-    case CPLXSXP:
-      complexes = COMPLEX(robj);
-      if (!(it = PyComplex_FromDoubles(complexes[i].r,
-                                       complexes[i].i)))
-        return -1;
-      break;
-    case STRSXP:
-      strings = CHAR(STRING_ELT(robj, i));
-      if (!(it = PyString_FromString(strings)))
-        return -1;
-      break;
-    case LISTSXP:
-      if (!(it = to_Pyobj_with_mode(elt(robj, i), 1)))
-        return -1;
-      break;
-    case VECSXP:
-      if (!(it = to_Pyobj_with_mode(VECTOR_ELT(robj, i), 1)))
-        return -1;
-      break;
-    default:
-      Py_DECREF(tmp);
-      return 0;                 /* failed */
+        break;
+      case STRSXP:
+        if(STRING_ELT(robj, i)==R_NaString)
+          it = PyString_FromString(CHAR(NA_STRING));
+        else
+          {
+            strings = CHAR(STRING_ELT(robj, i));
+            if (!(it = PyString_FromString(strings)))
+              return -1;
+          }
+        break;
+      case LISTSXP:
+        if (!(it = to_Pyobj_with_mode(elt(robj, i), mode)))
+          return -1;
+        break;
+      case VECSXP:
+        if (!(it = to_Pyobj_with_mode(VECTOR_ELT(robj, i), mode)))
+          return -1;
+        break;
+      default:
+        Py_DECREF(tmp);
+        return 0;                 /* failed */
     }
     
     if (PyList_SetItem(tmp, i, it) < 0)
@@ -584,14 +682,7 @@ to_Pyobj_basic(SEXP robj, PyObject **obj)
 
   names = GET_NAMES(robj);
   if (names == R_NilValue)
-    if (len == 1) {
-      *obj = PyList_GetItem(tmp, 0);
-      Py_XINCREF(*obj);
-      Py_DECREF(tmp);
-    } 
-    else {
-      *obj = tmp;
-    }
+    *obj = tmp;
   else {
     *obj = to_PyDict(tmp, names);
     Py_DECREF(tmp);
@@ -612,7 +703,7 @@ from_class_table(SEXP robj)
   fun = NULL;
   if (rclass != R_NilValue) {
 
-    lkey = to_Pyobj_with_mode(rclass, 1);
+    lkey = to_Pyobj_with_mode(rclass, BASIC_CONVERSION);
     key = PyList_AsTuple(lkey);
     if (key) {
       Py_DECREF(lkey);
@@ -696,7 +787,7 @@ to_Pyobj_proc(SEXP robj, PyObject **obj)
   return 1;                     /* conversion succeed */
 }
 
-/* Convert a Robj to a Python object via the class table (mode 2) */
+/* Convert a Robj to a Python object via the class table (mode 3) */
 /* See the docs for conversion rules */
 int
 to_Pyobj_class(SEXP robj, PyObject **obj)
@@ -721,21 +812,26 @@ to_Pyobj_with_mode(SEXP robj, int mode)
   PyObject *obj;
   int i;
 
-  switch (mode) {
-  case PROC_CONVERSION:
-    i = to_Pyobj_proc(robj, &obj);
-    if (i<0) return NULL;
-    if (i==1) break;
-  case CLASS_CONVERSION:
-    i = to_Pyobj_class(robj, &obj);
-    if (i<0) return NULL;
-    if (i==1) break;
-  case BASIC_CONVERSION:
-    i = to_Pyobj_basic(robj, &obj);
-    if (i<0) return NULL;
-    if (i==1) break;
-  default:
-    obj = (PyObject *)Robj_new(robj, TOP_MODE);
+  switch (mode)
+    {
+    case PROC_CONVERSION:
+      i = to_Pyobj_proc(robj, &obj);
+      if (i<0) return NULL;
+      if (i==1) break;
+    case CLASS_CONVERSION:
+      i = to_Pyobj_class(robj, &obj);
+      if (i<0) return NULL;
+      if (i==1) break;
+    case BASIC_CONVERSION:
+      i = to_Pyobj_basic(robj, &obj);
+      if (i<0) return NULL;
+      if (i==1) break;
+    case VECTOR_CONVERSION:
+      i = to_Pyobj_vector(robj, &obj, mode=VECTOR_CONVERSION);
+      if (i<0) return NULL;
+      if (i==1) break;
+    default:
+      obj = (PyObject *)Robj_new(robj, TOP_MODE);
   }
     
   return obj;
@@ -1031,7 +1127,7 @@ Robj_autoconvert(PyObject *self, PyObject *args, PyObject *kwds)
                                   &conversion))
     return NULL;
 
-  if (conversion > 3) {
+  if (conversion > TOP_MODE) {
     PyErr_SetString(PyExc_ValueError, "wrong mode");
     return NULL;
   }
@@ -1154,7 +1250,7 @@ Robj_item(PyObject *a, int i)
 
   /* If there is a default mode, use it; otherwise, use the top mode. */
   if (default_mode < 0)
-    c = 3;
+    c = TOP_MODE;
   else
     c = default_mode;
   obj = to_Pyobj_with_mode(robj, c);
@@ -1163,12 +1259,12 @@ Robj_item(PyObject *a, int i)
 
 /* We should implement sq_slice, sq_contains ... */
 static PySequenceMethods Robj_as_sequence = {
-  (inquiry)Robj_len,                            /* sq_length */
+  (inquiry)Robj_len,              /* sq_length */
   0,                            /* sq_concat */
   0,                            /* sq_repeat */
-  (intargfunc)Robj_item,                                /* sq_item */
+  (intargfunc)Robj_item,           /* sq_item */
   0,                            /* sq_slice */
-  (intobjargproc)Robj_ass_item,                         /* sq_ass_item */
+  (intobjargproc)Robj_ass_item,     /* sq_ass_item */
   0,                            /* sq_ass_slice */
   0,                            /* sq_contains */
 };
@@ -1185,7 +1281,7 @@ Robj_getattr(RobjObject *self, char *name)
 }
 #endif
 
-statichere PyTypeObject Robj_Type = {
+PyTypeObject Robj_Type = {
   /* The ob_type field must be initialized in the module init function
    * to be portable to Windows without using C++. */
 #if defined(PRE_2_2) || defined(_WIN32)    // Matjaz
@@ -1209,15 +1305,15 @@ statichere PyTypeObject Robj_Type = {
   0,                    /*tp_compare*/
   0,                    /*tp_repr*/
   0,                    /*tp_as_number*/
-  &Robj_as_sequence,                    /*tp_as_sequence*/
+  &Robj_as_sequence,    /*tp_as_sequence*/
   0,                    /*tp_as_mapping*/
   0,                    /*tp_hash*/
-  (ternaryfunc)Robj_call,                      /*tp_call*/
-  0,                      /*tp_str*/
+  (ternaryfunc)Robj_call,  /*tp_call*/
+  0,                    /*tp_str*/
 #if defined(PRE_2_2) || defined(_WIN32)
   0,
 #else
-  PyObject_GenericGetAttr,                      /*tp_getattro*/
+  PyObject_GenericGetAttr, /*tp_getattro*/
 #endif
   0,                      /*tp_setattro*/
   0,                      /*tp_as_buffer*/
@@ -1364,12 +1460,55 @@ start_events(void)
 }
 
 
-void
-r_cleanup(void)
+/*
+ * Based on code from Rstd_CleanUp();
+ * from src/unix/sys-std.c
+ */
+
+void r_finalize(void)
 {
-  R_CleanUp(SA_NOSAVE, 1, 0);
+    unsigned char buf[1024];
+    char * tmpdir;
+
+    R_dot_Last();           
+    R_RunExitFinalizers();  
+    CleanEd();              
+    KillAllDevices();       
+    if((tmpdir = getenv("R_SESSION_TMPDIR"))) {          
+      snprintf((char *)buf, 1024, "rm -rf %s", tmpdir); 
+      R_system((char *)buf);                            
+    }
+    if(R_CollectWarnings)
+      PrintWarnings();	/* from device close and .Last */
+                                                        
+    R_gc();  /* Remove any remaining R objects from memory */
 }
 
+  
+static PyObject *
+r_cleanup(void)
+{
+  r_cleanup();
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+static PyObject *
+r_init(void)
+{
+  static int first=1;
+  if(first==1)
+    {
+      first=0;
+      Py_INCREF(Py_None);
+      return Py_None;
+    }
+  else
+    {
+      PyErr_SetString(PyExc_RuntimeError, "Only one R object may be instantiated per session");
+      return NULL;
+    }
+}
 
 /* List of functions defined in the module */
 static PyMethodDef rpy_methods[] = {
@@ -1384,6 +1523,7 @@ static PyMethodDef rpy_methods[] = {
   {"get_showfiles", (PyCFunction)get_showfiles, METH_VARARGS},
   {"r_events",      (PyCFunction)r_events,      METH_VARARGS | METH_KEYWORDS},
   {"r_cleanup",     (PyCFunction)r_cleanup,     METH_NOARGS},
+  {"r_init",        (PyCFunction)r_init,        METH_NOARGS},
   {NULL, NULL}          /* sentinel */
 };
 
@@ -1486,9 +1626,16 @@ init_embedded_win32( void ) {
   Rp->WriteConsole = (blah2) RPy_WriteConsole;  // Matjaz
      
   Rp->CallBack = (blah3) RPyDoNothing;
+#if R_VERSION < 0x20100
   Rp->message = char_message;
   Rp->yesnocancel = char_yesnocancel;
   Rp->busy = RPyBusy;
+#else
+  Rp->ShowMessage = char_message;
+  Rp->YesNoCancel = char_yesnocancel;
+  Rp->Busy = RPyBusy;
+#endif
+
   Rp->R_Quiet = TRUE;
 
   /* run as "interactive", so server won't be killed after an error */
@@ -1515,10 +1662,9 @@ init_embedded_win32( void ) {
 #endif
 
 
-
 /* Initialization function for the module */
 DL_EXPORT(void)
-init_rpy(void)
+INIT_RPY(void)
 {
   static char RHome[] = R_HOME;
   PyObject *m, *d;
@@ -1539,7 +1685,7 @@ init_rpy(void)
   Robj_Type.tp_alloc = PyType_GenericAlloc;
 #endif
 
-  m = Py_InitModule("_rpy", rpy_methods);
+  m = Py_InitModule(RPY_SHNAME, rpy_methods);
 #ifdef WITH_NUMERIC
   import_array();
   multiarray = PyImport_ImportModule("multiarray");
@@ -1554,7 +1700,7 @@ init_rpy(void)
   /* Save this interpreter */
   PyEval_InitThreads();
   my_interp = PyThreadState_Get()->interp;
-
+  
   /* Save the Python signal handlers. If R inserts its handlers, we 
      cannot return to the Python interpreter. */
   old_int = PyOS_getsig(SIGINT);
@@ -1623,7 +1769,8 @@ init_rpy(void)
   R_interact = INTEGER(interact)[0];
 
   // I/O routines
-  init_io_routines(d);
+  init_io_routines();
+
 
 
   rpy = PyImport_ImportModule("rpy");
@@ -1631,6 +1778,14 @@ init_rpy(void)
   //  r_lock = PyDict_GetItemString(rpy_dict, "_r_lock");
   //  PyObject_Print(r_lock, stderr, Py_PRINT_RAW);
   r_lock = NULL;
+
+  if( Py_AtExit(	r_finalize ) )
+    {
+      fprintf(stderr, "Warning: Unable to set R finalizer.");
+      fflush(stderr);
+    }
+    
+      
 }
 
 
@@ -1658,3 +1813,4 @@ char *getRHOME(void)
   return buf;
 }
 #endif
+
