@@ -4,7 +4,6 @@ import unittest
 import random
 
 def run(module):
-    print 'Testing:', module[5:]
     try:
         unittest.main(module)
     except SystemExit:
@@ -16,8 +15,11 @@ if __name__ == '__main__':
     if '--random' in sys.argv:
         random.shuffle(modules)
         sys.argv.remove('--random')
+
+    modules = filter( lambda x: not x.endswith('.pyc'), modules)
+    modules = filter( lambda x: x.startswith('test_'), modules)
     
     for module in modules:
-        if module.startswith('test_') and not module.endswith('.pyc'):
-            name = module[:-3]
-            run(name)
+        print 'Testing:', module[5:-3]
+        name = module[:-3]
+        run(name)
